#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-

__author__ = "Y.Ban"

import re
from lxml import etree
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
import os.path
import urllib2
import shutil


class Writer(object):
    """Xml Converter Class"""

    def __init__(self, xmlpath, outputpath):
        self.buf = ""
        self._xmlpath = xmlpath
        self._output_path = outputpath
        name_re = re.compile('''^(.+)\.''')
        m = name_re.match(os.path.basename(outputpath))
        self._basename = m.group(1)

    def _img_download(self, url):
        try:
            img = urllib2.urlopen(url)
        except (urllib2.HTTPError, IOError):
            return None
        img_name = 'images/' + os.path.basename(url)
        localfile = open( self._output_dir + '/' + self._basename+'/images/'+os.path.basename(url), 'wb')
        localfile.write(img.read())
        img.close()
        localfile.close()
        return img_name

    def _packaging(self, root):
        image_list = root.xpath('//image')
        if os.path.dirname(self._output_path) == "":
            self._output_dir = '.'
        else:
            self._output_dir = os.path.dirname(self._output_path)
        print os.path.dirname(self._output_path)
        img_output_dir = self._output_dir + '/' + self._basename + '/images'
        try:
            os.makedirs(img_output_dir)
        except:
            print "makedir failed: " + img_output_dir
        for img in image_list:
            if img.get('uri').startswith('http://') or img.get('uri').startswith('https://'):
                download = self._img_download(img.get('uri'))
                if not download == None:
                    img.set('uri', download)
            else:
                try:
                    shutil.copy(os.path.dirname(self._xmlpath) + '/' + img.get('uri'), img_output_dir)
                except:
                    shutil.copy(img.get('uri'), img_output_dir)
                img.set('uri', 'images/'+ os.path.basename(img.get('uri')))
        self._output_path = self._output_dir + '/' + self._basename + '/' + os.path.basename(self._output_path)
        
        return root

    def parse_xml(self, pack=False):
        """Parse xml for lxml."""
        encoding = "UTF-8"
        fp = open(self._xmlpath, encoding)
        self._buf = fp.read()
        fp.close()
        self._buf = unicode(self._buf, 'utf-8')
        try:
            roots = etree.XML(self._buf)
        except:
            parser = etree.XMLParser(recover=True)
            roots = etree.fromstring(self._buf, parser=parser)
        if pack == True:
            roots = self._packaging(roots)
        return roots

    def _save(self, output):
        fp = open(self._output_path, "w")
        fp.write(output)
        fp.close()

    def export(self, output_text):
        """Export to a file."""
        if isinstance(output_text, unicode):
            self._save(output_text.encode("utf-8"))
        else:
            self._save(output_text)
